package xyz.lychee.lagfixer.libs.adventure.resource;

import java.net.URI;
import java.util.UUID;
import java.util.concurrent.CompletableFuture;
import java.util.concurrent.Executor;
import java.util.concurrent.ForkJoinPool;
import xyz.lychee.lagfixer.libs.adventure.builder.AbstractBuilder;
import xyz.lychee.lagfixer.libs.annotations.Contract;
import xyz.lychee.lagfixer.libs.annotations.NotNull;
import xyz.lychee.lagfixer.libs.examination.Examinable;

public interface ResourcePackInfo extends Examinable, ResourcePackInfoLike {
   static @NotNull ResourcePackInfo resourcePackInfo(final @NotNull UUID id, final @NotNull URI uri, final @NotNull String hash) {
      return new ResourcePackInfoImpl(id, uri, hash);
   }

   @NotNull
   static @NotNull Builder resourcePackInfo() {
      return new ResourcePackInfoImpl.BuilderImpl();
   }

   @NotNull UUID id();

   @NotNull URI uri();

   @NotNull String hash();

   default @NotNull ResourcePackInfo asResourcePackInfo() {
      return this;
   }

   public interface Builder extends AbstractBuilder<ResourcePackInfo>, ResourcePackInfoLike {
      @Contract("_ -> this")
      @NotNull
      @NotNull Builder id(final @NotNull UUID id);

      @Contract("_ -> this")
      @NotNull
      @NotNull Builder uri(final @NotNull URI uri);

      @Contract("_ -> this")
      @NotNull
      @NotNull Builder hash(final @NotNull String hash);

      @NotNull ResourcePackInfo build();

      default @NotNull CompletableFuture<ResourcePackInfo> computeHashAndBuild() {
         return this.computeHashAndBuild(ForkJoinPool.commonPool());
      }

      @NotNull CompletableFuture<ResourcePackInfo> computeHashAndBuild(final @NotNull Executor executor);

      default @NotNull ResourcePackInfo asResourcePackInfo() {
         return this.build();
      }
   }
}
